#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <string.h>

// microtime returns current time as double
double microtime() {
	int err;
	struct timeval tv;

	err = gettimeofday(&tv, NULL);
	if (err == -1) {
		perror("gettimeofday");
		abort();
	}

	return tv.tv_sec + 1E-6 * tv.tv_usec;
}

//		 12345678
#define	NITER	1000000

#define	BUFSIZE	64*1024
static char buf[BUFSIZE];
static char buf2[BUFSIZE];

int main() {
	int i;
	size_t size;

	for (size = 0; size <= BUFSIZE;) {
		// warmup
		memcpy(buf2, buf, size);

		double Tstart, Tend;
		Tstart = microtime();

		int niter = NITER;
		if (size > 10000)
			niter /= (size / 10000);	// compensate for larger bench sizes

		//for (i = 0; i < NITER; i++) {
		for (i = 0; i < niter; i++) {
			memcpy(buf2, buf, size);
		}

		Tend = microtime();

		printf("%ld\t%.1f\n", size, (Tend - Tstart) / niter / 1E-9);


		size += 128;

		//if (size < 4*1024) {
		//	size += 128;	// detailed start
		//} else if  (size < 32*1024) {
		//	size += 256;
		//} else {
		//	size += 512;
		//}
	}
}
