#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

//		 12345678
#define	NITER	1000000

// microtime returns current time as double
double microtime() {
	int err;
	struct timeval tv;

	err = gettimeofday(&tv, NULL);
	if (err == -1) {
		perror("gettimeofday");
		abort();
	}

	return tv.tv_sec + 1E-6 * tv.tv_usec;
}

#define	BUFSIZE	4096
static char buf[BUFSIZE];

// xpread is pread(2) but aborts on an error or when fewer bytes was read
void xpread(int fd, void *buf, size_t count, off_t offset) {
	ssize_t n;
	n = pread(fd, buf, count, offset);
	if (n == -1) {
		perror("pread");
		abort();
	}

	if (n != count) {
		fprintf(stderr, "pread(%ld) -> %ld\n", count, n);
		abort();
	}
}

int main(int argc, char *argv[]) {
	int fd, i;
	size_t size;

	if (argc < 2) {
		fprintf(stderr, "Usage: %s <file>\n", argv[0]);
		exit(1);
	}

	fd = open(argv[1], O_RDONLY);
	if (fd == -1) {
		perror("open");
		abort();
	}

	for (size = 0; size <= BUFSIZE;) {
		// warmup
		xpread(fd, &buf, size, /*offset=*/0);

		double Tstart, Tend;
		Tstart = microtime();

		for (i = 0; i < NITER; i++) {
			xpread(fd, buf, size, /*offset=*/0);
		}

		Tend = microtime();

		printf("%ld\t%.1f\n", size, (Tend - Tstart) / NITER / 1E-9);


		if (size < 16) {
			size += 1;	// detailed start
		} else if  (size < 128) {
			size += 16;
		} else {
			size += 128;
		}
	}
}
